/* FreeS/WAN comparisons functions (cmp.c)
 * Copyright (C) 2001-2002 Mathieu Lafon - Arkoon Network Security
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: cmp.c,v 1.4 2002/09/03 18:06:03 mathieu Exp $
 */

#include <string.h>

#include "confread.h"
#include "cmp.h"

#define streqn(a,b) (a)?((b)?(strcmp(a,b)):(-1)):(b!=NULL)

#define STRCMP(obj) if (streqn(c1->obj,c2->obj)) return -1
#define VARCMP(obj) if (c1->obj!=c2->obj) return -1
#define MEMCMP(obj) if (memcmp(&c1->obj,&c2->obj,sizeof(c1->obj))) return -1
#define ADDCMP(obj) if (addrcmp(&c1->obj,&c2->obj)) return -1
#define SUBCMP(obj) if (samesubnet(&c1->obj,&c2->obj)==0) return -1
#define LSTCMP(obj) if (cmp_list(c1->obj,c2->obj)!=0) return -1

static int cmp_list (char **l1, char **l2)
{
	if ((!l1) && (!l2)) return 0;
	if ((!l1) || (!l2)) return -1;
	for ( ; *l1 && *l2; l1++, l2++) {
		if (strcmp(*l1,*l2)) return -1;
	}
	if ((*l1) || (*l2)) return -1;
	return 0;
}

static int starter_cmp_end (struct starter_end *c1, struct starter_end *c2)
{
	if ((!c1) || (!c2)) return -1;
	VARCMP(addr_family);
	ADDCMP(addr);
	ADDCMP(nexthop);
	VARCMP(has_client);
	SUBCMP(subnet);
	STRCMP(iface);
	STRCMP(id);
	STRCMP(rsakey);
	STRCMP(updown);
#ifdef X509_PATCH
	STRCMP(cert);
	VARCMP(has_client_wildcard);
	VARCMP(port);
	VARCMP(protocol);
#endif
#ifdef VIRTUAL_IP
	STRCMP(virt);
#endif
	return 0;
}

int starter_cmp_conn (struct starter_conn *c1, struct starter_conn *c2)
{
	if ((!c1) || (!c2)) return -1;
	STRCMP(name);
	VARCMP(policy);
	VARCMP(sa_ike_life_seconds);
	VARCMP(sa_ipsec_life_seconds);
	VARCMP(sa_rekey_margin);
	VARCMP(sa_rekey_fuzz);
	VARCMP(sa_keying_tries);
	if (starter_cmp_end(&c1->left,&c2->left)) return -1;
	if (starter_cmp_end(&c1->right,&c2->right)) return -1;
	VARCMP(startup);
#ifdef ALG_PATCH
	STRCMP(esp);
	STRCMP(ike);
#endif
	return 0;
}

int starter_cmp_klips (struct starter_config *c1, struct starter_config *c2)
{
	if ((!c1) || (!c2)) return -1;
	LSTCMP(setup.klipsdebug);
	VARCMP(setup.fragicmp);
	STRCMP(setup.packetdefault);
	VARCMP(setup.hidetos);
	return 0;
}

int starter_cmp_pluto (struct starter_config *c1, struct starter_config *c2)
{
	if ((!c1) || (!c2)) return -1;
	LSTCMP(setup.plutodebug);
	STRCMP(setup.prepluto);
	STRCMP(setup.postpluto);
	VARCMP(setup.uniqueids);
#ifdef X509_PATCH
	VARCMP(setup.strictcrlpolicy);
	VARCMP(setup.nocrsend);
#endif
#ifdef NAT_TRAVERSAL
	VARCMP(setup.nat_traversal);
	VARCMP(setup.keep_alive);
#endif
#ifdef VIRTUAL_IP
	STRCMP(setup.virtual_private);
#endif
	return 0;
}

