/* FreeS/WAN config file parser (confread.h)
 * Copyright (C) 2001-2002 Mathieu Lafon - Arkoon Network Security
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: confread.h,v 1.4 2002/09/03 18:06:03 mathieu Exp $
 */

#ifndef _IPSEC_CONFREAD_H_
#define _IPSEC_CONFREAD_H_

#ifndef _OPENSWAN_H
#include <openswan.h>
#include "../pluto/constants.h"    /* FIXME: ugly include */
#endif

struct starter_end {
	sa_family_t addr_family;
	ip_address addr, nexthop;
	bool has_client;
	ip_subnet subnet;
	char *iface;
	char *id;
	char *rsakey;
	char *updown;
#ifdef X509_PATCH
    u_int16_t port;
    u_int8_t protocol;
	bool has_client_wildcard;
	char *cert;
	char *ca;
#endif
#ifdef VIRTUAL_IP
	char *virt;
#endif
};

struct starter_conn {
	char *name;

	lset_t policy;

	time_t sa_ike_life_seconds;
	time_t sa_ipsec_life_seconds;
	time_t sa_rekey_margin;
	unsigned long sa_rekey_fuzz;
	unsigned long sa_keying_tries;
	
	struct starter_end left, right;

	enum {
		STARTUP_NO,
		STARTUP_ADD,
		STARTUP_ROUTE,
		STARTUP_START
	} startup;

	unsigned long id;
	enum {
		STATE_INVALID,
		STATE_LOADED,
		STATE_TO_ADD,
		STATE_ADDED,
		STATE_REPLACED,
		STATE_IGNORE
	} state;

#ifdef ALG_PATCH
	char *esp;
	char *ike;
#endif
#ifdef DPD
		time_t dpd_delay;
		time_t dpd_timeout;
		int dpd_action;
#endif

	struct starter_conn *next;
};

struct starter_config {
	struct {
		char **interfaces;
		/* TODO: bool forwardcontrol; */
		/* TODO: char *syslog; */
		char **klipsdebug;
		char **plutodebug;
		/* TODO: char *dumpdir; */
		/* TODO: char *manualstart; */
		/* TODO: bool pluto; */
		/* TODO: char *plutoload; */
		/* TODO: char *plutostart; */
		/* TODO: bool plutowait; */
		char *prepluto;
		char *postpluto;
		bool fragicmp;
		char *packetdefault;
		bool hidetos;
		bool uniqueids;
		unsigned int overridemtu;
#ifdef X509_PATCH
		bool strictcrlpolicy;
		bool nocrsend;
#endif
#ifdef NAT_TRAVERSAL
		bool nat_traversal;
		unsigned int keep_alive;
#endif
#ifdef VIRTUAL_IP
		char *virtual_private;
#endif
	} setup;

	/* conn %default */
	struct starter_conn conn_default;

	/* connections list (without %default) */
	struct starter_conn *conn_first, *conn_last;
};

struct starter_config *confread_load(const char *file, char **perr);

void confread_free(struct starter_config *cfg);

#endif /* _IPSEC_CONFREAD_H_ */

