/* FreeS/WAN IPsec starter (starter.c)
 * Copyright (C) 2001-2002 Mathieu Lafon - Arkoon Network Security
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * RCSID $Id: klips.c,v 1.2 2002/09/03 18:06:03 mathieu Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>

#include "confread.h"
#include "klips.h"
#include "files.h"
#include "log.h"
#include "exec.h"

static int _klips_module_loaded = 0;

void starter_klips_clear (void)
{
	system(IPSEC_LIB_DIR"/eroute --clear");
	system(IPSEC_LIB_DIR"/spi --clear");
	system(IPSEC_LIB_DIR"/klipsdebug --none");
}

void starter_klips_cleanup (void)
{
	starter_klips_clear();
	if (_klips_module_loaded) {
		system("rmmod ipsec");
		_klips_module_loaded = 0;
	}
}

static void _sysflags (char *name, int value)
{
	if (starter_exec(
		"echo %d >%s/%s 2>/dev/null", value?1:0, PROC_SYSFLAGS, name
		)!=0) {
		starter_log(LOG_LEVEL_ERR, "can't set sysflag %s to %d", name,
			value ? 1 : 0);
	}
}

int starter_klips_set_config (struct starter_config *cfg)
{
	char **l;

	_sysflags("icmp", cfg->setup.fragicmp);
	_sysflags("inbound_policy_check", 1);
	//_sysflags("no_eroute_pass", 0);
	//_sysflags("opportunistic", 0);
	_sysflags("tos", cfg->setup.hidetos);

	starter_exec("%s/klipsdebug --none", IPSEC_LIB_DIR);
	for (l=cfg->setup.klipsdebug; l && *l; l++) {
		if ((strcmp(*l,"none")==0) || (strcmp(*l,"all")==0))
			starter_exec("%s/klipsdebug --%s", IPSEC_LIB_DIR, *l);
		else
			starter_exec("%s/klipsdebug --set %s", IPSEC_LIB_DIR, *l);
	}

	starter_exec("%s/eroute --del --eraf inet --src 0/0 --dst 0/0 2>/dev/null",
		IPSEC_LIB_DIR);
	starter_exec("%s/eroute --label packetdefault --replace --eraf inet "
		"--src 0/0 --dst 0/0 --said %%%s", IPSEC_LIB_DIR,
		cfg->setup.packetdefault ? cfg->setup.packetdefault : "drop");

	return 0;
}

int starter_klips_init (void)
{
	struct stat stb;

	if (stat(PROC_IPSECVERSION,&stb)!=0) {
		if (stat(PROC_MODULES,&stb)==0) {
			unsetenv("MODPATH");
			unsetenv("MODULECONF");
			system("depmod -a >/dev/null 2>&1 && modprobe ipsec");
		}
		if (stat(PROC_IPSECVERSION,&stb)==0) {
			_klips_module_loaded = 1;
		}
		else {
			starter_log(LOG_LEVEL_ERR, "kernel appears to lack KLIPS");
			return 1;
		}
	}

	starter_klips_clear();

	return 0;
}

